<?php

namespace Drupal\cookiebot\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Path\AliasManager;

/**
 * Cookiebot settings form.
 */
class CookiebotForm extends ConfigFormBase {

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   *   The entity type manager.
   */
  protected $entityTypeManager;


  /**
   * Alias manager.
   *
   * @var \Drupal\Core\Path\AliasManager
   *   The Alias Manager.
   */
  protected $aliasManager;

  /**
   * Constructs a object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_manager
   *   The entity type manager.
   * @param \Drupal\Core\Path\AliasManager $alias_manager
   *   The alias manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManager $entity_manager, AliasManager $alias_manager) {
    parent::__construct($config_factory);
    $this->setConfigFactory($config_factory);
    $this->entityTypeManager = $entity_manager;
    $this->aliasManager = $alias_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('path.alias_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'cookiebot.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cookiebot_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('cookiebot.settings');
    $form['cookiebot_cbid'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Your cookiebot CBID'),
      '#description' => $this->t('Id looks like 00000000-0000-0000-0000-000000000000. You can find it in your <a href="https://www.cookiebot.com/en/manage">Cookiebot Manager</a>.'),
      '#default_value' => $config->get('cookiebot_cbid'),
    ];

    $form['cookiebot_declaration'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Cookie Declaration'),
    ];

    $form['cookiebot_declaration']['cookiebot_allow_declaration'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allow Cookiebot declaration'),
      '#description' => $this->t('Display the full Cookie Declaration on subpage'),
      '#default_value' => $config->get('cookiebot_allow_declaration'),
    ];

    $declaration = '';
    $alias = $this->aliasManager->getPathByAlias($config->get('cookiebot_declaration_path'));
    if (preg_match('/node\/(\d+)/', $alias, $matches)) {
      $declaration = $this->entityTypeManager->getStorage('node')->load($matches[1]);
    }

    $form['cookiebot_declaration']['cookiebot_declaration_path'] = [
      '#type' => 'entity_autocomplete',
      '#target_type' => 'node',
      '#title' => $this->t('Full declaration path'),
      '#description' => $this->t('Relative path, where you can display full Cookie Declaration'),
      '#default_value' => $declaration,
      '#states' => [
        'visible' => [
          ':input[name="cookiebot_allow_declaration"]' => [
            'checked' => TRUE,
          ],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $alias = $this->aliasManager->getAliasByPath('/node/' . $form_state->getValue('cookiebot_declaration_path'));
    $this->config('cookiebot.settings')
      ->set('cookiebot_cbid', $form_state->getValue('cookiebot_cbid'))
      ->set('cookiebot_allow_declaration', $form_state->getValue('cookiebot_allow_declaration'))
      ->set('cookiebot_declaration_path', $alias)
      ->save();
  }

}
